VERSION 4.00
Begin VB.Form Form1 
   Caption         =   "Regions sample by Matt Hart - vbhelp@matthart.com"
   ClientHeight    =   2985
   ClientLeft      =   1725
   ClientTop       =   1935
   ClientWidth     =   6480
   Height          =   3390
   Left            =   1665
   LinkTopic       =   "Form1"
   ScaleHeight     =   2985
   ScaleWidth      =   6480
   Top             =   1590
   Width           =   6600
   Begin VB.Label Label1 
      Caption         =   "Move the mouse over the yellow region for movement detection."
      Height          =   915
      Index           =   1
      Left            =   4860
      TabIndex        =   1
      Top             =   1920
      Width           =   1455
   End
   Begin VB.Label Label1 
      Caption         =   "Click on a region graphic for hit detection."
      Height          =   915
      Index           =   0
      Left            =   180
      TabIndex        =   0
      Top             =   1860
      Width           =   1455
   End
End
Attribute VB_Name = "Form1"
Attribute VB_Creatable = False
Attribute VB_Exposed = False


Option Explicit

' Regions sample by Matt Hart - vbhelp@matthart.com
' http://matthart.com
'
' This sample shows how to create regions and detect mouse
' movements and clicks. You can also easily detect collisions
' between regions.
' Quite a few APIs are used here, and I highly recommend reading
' their descriptions in the SDK.
'
' Note that I repaint the regions in the Paint event. You can
' also keep regions by setting .AutoRedraw = True before
' painting the regions. The best way is to subclass the Form
' and monitor the WM_PAINT event, then use GetUpdateRect to
' see what part of the form needs updating, along with the
' RectInRegion API to see which regions are within that
' rectangle. To move a region, use the OffsetRegion API
' function.
'
' You'll generally use the GetRgnBox API along with the
' RectInRegion API to detect collisions between regions.
' Get a region's rectangle with GetRgnBox and see if it is
' hitting a point in another region with RectInRegion.
'
' You can also combine regions to create more complex shapes
' with better detection. For example, a creature with a long
' arm would be a good candidate for combining two regions - one
' for the body and one for the arm. Then, when you wanted to detect
' the arm hitting something, your GetRgnBox would be more accurate
' with just the arm to work with rather than the whole body of the
' creature.

Private Type LOGBRUSH
    lbStyle As Long
    lbColor As Long
    lbHatch As Long
End Type
Private Type POINTAPI
    X As Long
    Y As Long
End Type
Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type
Private Declare Function CreateBrushIndirect Lib "gdi32" (lpLogBrush As LOGBRUSH) As Long
Private Declare Function CreateEllipticRgn Lib "gdi32" (ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Private Declare Function CreatePen Lib "gdi32" (ByVal nPenStyle As Long, ByVal nWidth As Long, ByVal crColor As Long) As Long
Private Declare Function CreatePolygonRgn Lib "gdi32" (lpPoint As POINTAPI, ByVal nCount As Long, ByVal nPolyFillMode As Long) As Long
Private Declare Function CreateRectRgn Lib "gdi32" (ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Private Declare Function CreateRoundRectRgn Lib "gdi32" (ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long, ByVal X3 As Long, ByVal Y3 As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function GetCurrentObject Lib "gdi32" (ByVal hdc As Long, ByVal uObjectType As Long) As Long
Private Declare Function PaintRgn Lib "gdi32" (ByVal hdc As Long, ByVal hRgn As Long) As Long
Private Declare Function PtInRegion Lib "gdi32" (ByVal hRgn As Long, ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function ReleaseCapture Lib "user32" () As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function SetCapture Lib "user32" (ByVal hwnd As Long) As Long

' Brush Styles
Private Const BS_SOLID = 0
Private Const BS_NULL = 1
Private Const BS_HOLLOW = BS_NULL
Private Const BS_HATCHED = 2
Private Const BS_PATTERN = 3
Private Const BS_INDEXED = 4
Private Const BS_DIBPATTERN = 5
Private Const BS_DIBPATTERNPT = 6
Private Const BS_PATTERN8X8 = 7
Private Const BS_DIBPATTERN8X8 = 8

'  Hatch Styles
Private Const HS_HORIZONTAL = 0              '  -----
Private Const HS_VERTICAL = 1                '  |||||
Private Const HS_FDIAGONAL = 2               '  \\\\\
Private Const HS_BDIAGONAL = 3               '  /////
Private Const HS_CROSS = 4                   '  +++++
Private Const HS_DIAGCROSS = 5               '  xxxxx
Private Const HS_FDIAGONAL1 = 6
Private Const HS_BDIAGONAL1 = 7
Private Const HS_SOLID = 8
Private Const HS_DENSE1 = 9
Private Const HS_DENSE2 = 10
Private Const HS_DENSE3 = 11
Private Const HS_DENSE4 = 12
Private Const HS_DENSE5 = 13
Private Const HS_DENSE6 = 14
Private Const HS_DENSE7 = 15
Private Const HS_DENSE8 = 16
Private Const HS_NOSHADE = 17
Private Const HS_HALFTONE = 18
Private Const HS_SOLIDCLR = 19
Private Const HS_DITHEREDCLR = 20
Private Const HS_SOLIDTEXTCLR = 21
Private Const HS_DITHEREDTEXTCLR = 22
Private Const HS_SOLIDBKCLR = 23
Private Const HS_DITHEREDBKCLR = 24
Private Const HS_API_MAX = 25

'  Pen Styles
Private Const PS_SOLID = 0
Private Const PS_DASH = 1                    '  -------
Private Const PS_DOT = 2                     '  .......
Private Const PS_DASHDOT = 3                 '  _._._._
Private Const PS_DASHDOTDOT = 4              '  _.._.._
Private Const PS_NULL = 5
Private Const PS_INSIDEFRAME = 6
Private Const PS_USERSTYLE = 7
Private Const PS_ALTERNATE = 8
Private Const PS_STYLE_MASK = &HF

Private Const PS_ENDCAP_ROUND = &H0
Private Const PS_ENDCAP_SQUARE = &H100
Private Const PS_ENDCAP_FLAT = &H200
Private Const PS_ENDCAP_MASK = &HF00

Private Const PS_JOIN_ROUND = &H0
Private Const PS_JOIN_BEVEL = &H1000
Private Const PS_JOIN_MITER = &H2000
Private Const PS_JOIN_MASK = &HF000

Private Const PS_COSMETIC = &H0
Private Const PS_GEOMETRIC = &H10000
Private Const PS_TYPE_MASK = &HF0000

Private Const AD_COUNTERCLOCKWISE = 1
Private Const AD_CLOCKWISE = 2

' PolyFill() Modes
Private Const ALTERNATE = 1
Private Const WINDING = 2
Private Const POLYFILL_LAST = 2

'  Object Definitions for EnumObjects()
Private Const OBJ_PEN = 1
Private Const OBJ_BRUSH = 2
Private Const OBJ_DC = 3
Private Const OBJ_METADC = 4
Private Const OBJ_PAL = 5
Private Const OBJ_FONT = 6
Private Const OBJ_BITMAP = 7
Private Const OBJ_REGION = 8
Private Const OBJ_METAFILE = 9
Private Const OBJ_MEMDC = 10
Private Const OBJ_EXTPEN = 11
Private Const OBJ_ENHMETADC = 12
Private Const OBJ_ENHMETAFILE = 13

Dim hRegion() As Long, lRegions As Long
Dim hBrush() As Long, hPen() As Long
Dim hPenSave As Long, hBrushSave As Long
Dim bOver As Boolean

Private Sub Form_Load()
    hPenSave = GetCurrentObject(hdc, OBJ_PEN)
    hBrushSave = GetCurrentObject(hdc, OBJ_BRUSH)
    Dim nCount As Long, pi As Single, s As Single, LB As LOGBRUSH
    lRegions = 6
    ReDim hRegion(1 To lRegions) As Long
    ReDim hBrush(1 To lRegions) As Long
    ReDim hPen(1 To lRegions) As Long
    pi = 3.141592654
    nCount = pi * 2 / 0.3 + 1 ' .3 = resolution - the lower the number, the higher the resolution
    ReDim P(1 To nCount) As POINTAPI
    nCount = 0
    For s = 0 To pi * 2 Step 0.3
        nCount = nCount + 1
        P(nCount).X = 50 + Cos(s) * 25
        P(nCount).Y = 30 + Sin(s) * 25
    Next
    hRegion(1) = CreatePolygonRgn(P(1), nCount, WINDING)
    hRegion(2) = CreateEllipticRgn(90, 10, 200, 60)
    hRegion(3) = CreateRectRgn(225, 10, 290, 50)
    hRegion(4) = CreateRoundRectRgn(310, 10, 400, 40, 10, 10)
    
    nCount = pi * 2 / (pi * 2 / 4) + 1
    ReDim P(1 To nCount) As POINTAPI
    nCount = 0
    For s = 0 To pi * 2 Step (pi * 2 / 4)
        nCount = nCount + 1
        P(nCount).X = 200 + Cos(s) * 40
        P(nCount).Y = 140 + Sin(s) * 40
    Next
    hRegion(5) = CreatePolygonRgn(P(1), nCount, WINDING)
    
    hPen(1) = CreatePen(PS_SOLID, 4, QBColor(1))
    LB.lbColor = QBColor(1)
    LB.lbStyle = BS_HATCHED
    LB.lbHatch = HS_DIAGCROSS
    hBrush(1) = CreateBrushIndirect(LB)
    
    LB.lbColor = QBColor(2)
    LB.lbHatch = HS_BDIAGONAL
    hPen(2) = hPen(1)
    hBrush(2) = CreateBrushIndirect(LB)
    
    LB.lbColor = QBColor(3)
    LB.lbStyle = BS_SOLID
    LB.lbHatch = HS_SOLID
    hPen(3) = hPen(2)
    hBrush(3) = CreateBrushIndirect(LB)
    
    LB.lbColor = QBColor(3)
    LB.lbStyle = BS_SOLID
    LB.lbHatch = HS_SOLID
    hPen(3) = hPen(2)
    hBrush(3) = CreateBrushIndirect(LB)
    
    LB.lbColor = QBColor(5)
    LB.lbStyle = BS_HATCHED
    LB.lbHatch = HS_FDIAGONAL
    hPen(4) = hPen(3)
    hBrush(4) = CreateBrushIndirect(LB)
    
    LB.lbColor = QBColor(13)
    LB.lbStyle = BS_SOLID
    LB.lbHatch = HS_SOLID
    hPen(5) = hPen(4)
    hBrush(5) = CreateBrushIndirect(LB)
    
    ReDim P(1 To 4) As POINTAPI
    P(1).X = 10: P(1).Y = 80
    P(2).X = 400: P(2).Y = 90
    P(3).X = 401: P(3).Y = 91
    P(4).X = 11: P(4).Y = 81
    hRegion(6) = CreatePolygonRgn(P(1), 4, WINDING)
    LB.lbColor = QBColor(14)
    LB.lbStyle = BS_SOLID
    LB.lbHatch = HS_SOLID
    hPen(6) = CreatePen(PS_SOLID, 0, QBColor(14))
    hBrush(6) = CreateBrushIndirect(LB)
End Sub

Private Sub Form_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    Dim k As Long
    For k = 1 To lRegions
        If PtInRegion(hRegion(k), X \ Screen.TwipsPerPixelX, Y \ Screen.TwipsPerPixelY) Then MsgBox "Region " & k: Exit For
    Next
End Sub

Private Sub Form_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If PtInRegion(hRegion(5), X \ Screen.TwipsPerPixelX, Y \ Screen.TwipsPerPixelY) Then
        If Not bOver Then
            SetCapture hwnd
            bOver = True
            SelectObject hdc, hBrush(1)
            PaintRgn hdc, hRegion(5)
            SelectObject hdc, hBrushSave
        End If
    ElseIf bOver Then
        ReleaseCapture
        bOver = False
        SelectObject hdc, hBrush(5)
        PaintRgn hdc, hRegion(5)
        SelectObject hdc, hBrushSave
    End If
        
End Sub

Private Sub Form_Paint()
    Dim k As Long
    For k = 1 To lRegions
        SelectObject hdc, hPen(k)
        SelectObject hdc, hBrush(k)
        PaintRgn hdc, hRegion(k)
    Next
    SelectObject hdc, hPenSave
    SelectObject hdc, hBrushSave
End Sub

Private Sub Form_Unload(Cancel As Integer)
    Dim k As Long
    For k = 1 To lRegions
        DeleteObject hRegion(k)
        DeleteObject hPen(k)
        DeleteObject hBrush(k)
    Next
End Sub
